from confluence.models.user import User
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def test_creation_valid_json():
    u = User({
        'username': '1',
        'displayName': '2',
        'userKey': '3',
        'type': '4'
    })
    assert u.username == '1'
    assert u.display_name == '2'
    assert u.user_key == '3'
    assert u.type == '4'

    assert str(u) == '1'


def test_creation_minimal_json():
    u = User({})
    assert u.username is None
    assert not hasattr(u, 'display_name')
    assert not hasattr(u, 'user_key')
    assert not hasattr(u, 'type')

    assert str(u) == 'None'



from confluence.models.page import Page
from datetime import date
import logging
import requests
from typing import Callable, Dict, Iterable, Optional, Tuple

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Confluence:
    """
    External interface into this library, all calls should be made through
    an instance of this class.
    """

    def __init__(self, base_url: str, basic_auth: Tuple[str, str]) -> None:
        """

        :param base_url: The URL where the confluence web app is located. e.g. https://mysite.mydomain/confluence
        :param basic_auth: A tuple containing a username/password pair that
        can log into confluence.
        """
        self._base_url = base_url
        self._basic_auth = basic_auth

        self._api_base = f'{self._base_url}/rest/api'

    def _get_paged_results(self, item_type: Callable, url: str, params: Dict[str, str],):
        while url is not None:
            search_results = requests.get(url, params=params, auth=self._basic_auth).json()

            if 'next' in search_results['_links']:
                # We have another page of results
                url = f"{self._base_url}{search_results['_links']['next']}"
                params.clear()
            else:
                # No more pages of results
                url = None

            for result in search_results['results']:
                yield item_type(result)

    def get_content(self, content_type: str = 'page', space_key: Optional[str] = None,
                    title: Optional[str] = None, status: Optional[str] = None, posting_day: Optional[date] = None,
                    expand: Optional[str] = None) -> Iterable[Page]:
        """
        Matches the REST API call https://docs.atlassian.com/atlassian-confluence/REST/6.6.0/#content-getContent
        which returns an iterable of either pages or blogposts depending on
        the value of the content_type parameter. The default is to return documents.

        Note that this function handles pagination automatically and returns
        an iterable containing all content. Therefore any attempt to
        materialise the results will result in potentially large numbers of
        HTTP requests.

        :param content_type: Determines whether we want to return blog posts
        of pages, defaults to page. Valid values are page|blogpost.
        :param space_key: The string space key of a space on the confluence
        server. Defaults to None which results in this field being ignored.
        :param title: The title of the page we're looking for. Defaults to
        None which results in this field being ignored.
        :param status: Only return documents in a given status.
        Defaults to None which results in this field being ignored.
        :param posting_day: Only valid for blogpost content_type and returns
        blogs posted on the given day.
        :param expand: The confluence REST API utilised expansion to avoid
        returning all fields on all requests. This optional parameter allows
        the user to select which fields that they want to expand as a comma
        separated list.

        :return: An iterable of pages/blogposts which match the parameters.
        """
        content_url = f'{self._api_base}/content'
        params = {}
        if content_type and content_type in ('page', 'blogpost'):
            params['type'] = content_type
        if space_key:
            params['spaceKey'] = space_key
        if title:
            params['title'] = title
        if status:
            params['status'] = status
        if posting_day and content_type == 'blogpost':
            params['postingDay'] = posting_day.strftime('%Y-%m-%d')
        if expand:
            params['expand'] = expand

        return self._get_paged_results(Page, content_url, params)

    def search(self, cql: str, cql_context: Optional[str] = None, expand: Optional[str] = None) -> Iterable[Page]:
        """
        Perform a CQL search on the confluence instance and return an iterable
        of the pages which match the query.

        :param cql: A CQL query. See https://developer.atlassian.com/server/confluence/advanced-searching-using-cql/
        for reference.
        :param cql_context: "the context to execute a cql search in, this is
        the json serialized form of SearchContext".
        :param expand: The confluence REST API utilised expansion to avoid
        returning all fields on all requests. This optional parameter allows
        the user to select which fields that they want to expand as a comma
        separated list.

        :return: An iterable of pages which match the parameters.
        """
        search_url = f'{self._api_base}/content/search'
        params = {'cql': cql}
        if cql_context:
            params['cqlcontext'] = cql_context
        if expand:
            params['expand'] = expand

        return self._get_paged_results(Page, search_url, params)

    def __str__(self):
        return self._api_base



from confluence.models.user import User
import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class PageUpdate:
    """
    Represents the information stored in confluence about a single page update.
    """
    def __init__(self, json: Dict[str, Any]) -> None:
        self.updater = User(json['by'])
        self.when = json['when']
        self.message = json['message']
        self.number = json['number']
        self.minor_edit = json['minorEdit']
        self.hidden = json['hidden']

    def __str__(self):
        return f'{self.updater} - {self.when}'



import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class User:
    """
    Represents a single user object in confluence either as attached to a page
    or as requested directly over the API.

    Note that all fields here are optional as the user attached to pages might
    not contain standard fields (e.g. when it's anonymous)
    """

    def __init__(self, json: Dict[str, Any]) -> None:
        # Fields are not always present when requesting a user.
        self.username = json['username'] if 'username' in json else None
        if 'displayName' in json:
            self.display_name = json['displayName']
        if 'userKey' in json:
            self.user_key = json['userKey']
        if 'type' in json:
            self.type = json['type']

    def __str__(self):
        return f'{self.username}'



from confluence.models.pageupdate import PageUpdate
from confluence.models.space import Space
from confluence.models.user import User
import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Page:
    """
    Represents a single page in Confluence. Note that fields are only
    populated if the corresponding area was expanded in the HTTP request.
    c.f. https://developer.atlassian.com/server/confluence/expansions-in-the-rest-api/
    for more details.
    """

    def __init__(self, json: Dict[str, Any]) -> None:
        # Fields that must exist on the json object
        self.id = json['id']
        self.title = json['title']
        self.status = json['status']

        # Fields that only exist if the space is expanded
        if 'space' in json:
            self.space = Space(json['space'])

        # Fields that only exist if a part of the body is expanded
        if 'body' in json:
            if 'storage' in json['body']:
                self.body_storage = json['body']['storage']
            if 'editor' in json['body']:
                self.body_storage = json['body']['editor']
            if 'view' in json['body']:
                self.body_storage = json['body']['view']
            if 'export_view' in json['body']:
                self.body_storage = json['body']['export_view']
            if 'styled_view' in json['body']:
                self.body_storage = json['body']['styled_view']
            if 'anonymous_export_view' in json['body']:
                self.body_storage = json['body']['anonymous_export_view']

        # Fields that only exist if the history is expanded
        if 'history' in json:
            self.latest = json['history']['latest']
            self.created_date = json['history']['createdDate']
            self.author = User(json['history']['createdBy'])

            # Fields only returned if the history.lastUpdated is expanded
            self.last_updated = PageUpdate(json['history']['lastUpdated'])

    def __str__(self):
        return f'{self.id} - {self.title}'



from setuptools import setup, find_packages

setup(
    name='confluence-rest-library',
    packages=find_packages(),
    version='0.2.2',
    description='A simple wrapper around the Confluence REST API.',
    author='David Tyler',
    author_email='davet.code@gmail.com',
    url='https://github.com/DaveTCode/confluence-python-lib',
    keywords=['confluence'],
    classifiers=[],
    python_requires='>=3.6',
    setup_requires=['pytest-runner'],
    install_requires=['requests >= 2.18.4, < 3.0.0a0'],
    tests_require=['pytest >= 3.0.7, < 4.0.0a0', 'pytest-cov >= 2.5.0, < 3.0.0a0']
)



import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Space:
    """
    Represents a single space in Confluence. Spaces have no optional/
    expandable content.
    """

    def __init__(self, json: Dict[str, Any]) -> None:
        # All fields always exist on the json object
        self.id = json['id']
        self.key = json['key']
        self.name = json['name']
        self.type = json['type']

    def __str__(self):
        return f'{self.id} - {self.key} | {self.name}'
